@@ -35,9 +35,9 @@ from pytket.passes import (  # type: ignore
     FlattenRegisters,
     RebaseCustom,
     EulerAngleReduction,
-    RemoveRedundancies,
     DecomposeBoxes,
     RenameQubitsPass,
+    SimplifyInitial,
 )
 from pytket.predicates import (  # type: ignore
     GateSetPredicate,
@@ -49,6 +49,7 @@ from pytket.predicates import (  # type: ignore
     Predicate,
 )
 from pytket.routing import FullyConnected  # type: ignore
+from pytket.utils import prepare_circuit
 from pytket.utils.outcomearray import OutcomeArray
 from .config import AQTConfig
 
@@ -93,6 +94,7 @@ class AQTBackend(Backend):
 
     _supports_shots = True
     _supports_counts = True
+    _supports_contextual_optimisation = True
     _persistent_handles = True
 
     def __init__(
@@ -174,7 +176,9 @@ class AQTBackend(Backend):
                     FlattenRegisters(),
                     RenameQubitsPass(self._qm),
                     _aqt_rebase(),
-                    RemoveRedundancies(),
+                    SimplifyInitial(
+                        allow_classical=False, create_all_qubits=True, xcirc=_xcirc
+                    ),
                     EulerAngleReduction(OpType.Ry, OpType.Rx),
                 ]
             )
@@ -186,14 +190,16 @@ class AQTBackend(Backend):
                     FlattenRegisters(),
                     RenameQubitsPass(self._qm),
                     _aqt_rebase(),
-                    RemoveRedundancies(),
+                    SimplifyInitial(
+                        allow_classical=False, create_all_qubits=True, xcirc=_xcirc
+                    ),
                     EulerAngleReduction(OpType.Ry, OpType.Rx),
                 ]
             )
 
     @property
     def _result_id_type(self) -> _ResultIdTuple:
-        return (str, str)
+        return (str, str, str)
 
     def process_circuits(
         self,
@@ -211,13 +217,23 @@ class AQTBackend(Backend):
 
         if valid_check:
             self._check_all_circuits(circuits)
+
+        postprocess = kwargs.get("postprocess", False)
+
         handles = []
         for i, c in enumerate(circuits):
-            (aqt_circ, measures) = _translate_aqt(c)
+            if postprocess:
+                c0, ppcirc = prepare_circuit(c, allow_classical=False, xcirc=_xcirc)
+                ppcirc_rep = ppcirc.to_dict()
+            else:
+                c0, ppcirc_rep = c, None
+            (aqt_circ, measures) = _translate_aqt(c0)
             if self._MACHINE_DEBUG:
                 handles.append(
                     ResultHandle(
-                        _DEBUG_HANDLE_PREFIX + str((c.n_qubits, n_shots)), measures
+                        _DEBUG_HANDLE_PREFIX + str((c.n_qubits, n_shots)),
+                        measures,
+                        json.dumps(ppcirc_rep),
                     )
                 )
             else:
@@ -235,7 +251,9 @@ class AQTBackend(Backend):
                     raise RuntimeError(resp["message"])
                 if resp["status"] == "error":
                     raise RuntimeError(resp["ERROR"])
-                handles.append(ResultHandle(resp["id"], measures))
+                handles.append(
+                    ResultHandle(resp["id"], measures, json.dumps(ppcirc_rep))
+                )
         for handle in handles:
             self._cache[handle] = dict()
         return handles
@@ -245,10 +263,14 @@ class AQTBackend(Backend):
         jobid = handle[0]
         message = ""
         measure_permutations = json.loads(handle[1])  # type: ignore
+        ppcirc_rep = json.loads(cast(str, handle[2]))
+        ppcirc = Circuit.from_dict(ppcirc_rep) if ppcirc_rep is not None else None
         if self._MACHINE_DEBUG:
             n_qubits, n_shots = literal_eval(jobid[len(_DEBUG_HANDLE_PREFIX) :])  # type: ignore
             empty_ar = OutcomeArray.from_ints([0] * n_shots, n_qubits, big_endian=True)
-            self._cache[handle].update({"result": BackendResult(shots=empty_ar)})
+            self._cache[handle].update(
+                {"result": BackendResult(shots=empty_ar, ppcirc=ppcirc)}
+            )
             statenum = StatusEnum.COMPLETED
         else:
             data = put(self._url, data={"id": jobid}, headers=self._header).json()
@@ -261,7 +283,9 @@ class AQTBackend(Backend):
                     data["samples"], data["no_qubits"], big_endian=True
                 )
                 shots = shots.choose_indices(measure_permutations)
-                self._cache[handle].update({"result": BackendResult(shots=shots)})
+                self._cache[handle].update(
+                    {"result": BackendResult(shots=shots, ppcirc=ppcirc)}
+                )
         return CircuitStatus(statenum, message)
 
     def get_result(self, handle: ResultHandle, **kwargs: KwargTypes) -> BackendResult:
@@ -329,3 +353,7 @@ def _aqt_rebase() -> BasePass:
     c_tk1 = lambda a, b, c: Circuit(1).Rx(-0.5, 0).Ry(c, 0).Rx(b, 0).Ry(a, 0).Rx(0.5, 0)
 
     return RebaseCustom({OpType.XXPhase}, c_cx, {OpType.Rx, OpType.Ry}, c_tk1)
+
+
+_xcirc = Circuit(1).Rx(1, 0)
+_xcirc.add_phase(0.5)
